=== modified file 'NEWS'
--- NEWS	2009-02-27 05:58:29 +0000
+++ NEWS	2009-02-27 08:29:09 +0000
@@ -90,6 +90,11 @@
       for branches. This is now fixed and a test added to prevent it
       reoccuring. (Robert Collins, Andrew Bennetts)
 
+    * Under rare circumstances (aka nobody reported a bug about it), the ftp
+      transport could revert to ascii mode. It now stays in binary mode except
+      when needed.
+      (Vincent Ladeuil)
+
   DOCUMENTATION:
 
     * The documentation for ``shelve`` and ``unshelve`` has been clarified.

=== modified file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	2009-01-17 01:30:58 +0000
+++ bzrlib/tests/test_transport_implementations.py	2009-02-26 19:30:06 +0000
@@ -1115,6 +1115,7 @@
 
         self.assertListRaises(PathError, t.list_dir, 'q')
         self.assertListRaises(PathError, t.list_dir, 'c/f')
+        # 'a' is a file, list_dir should raise an error
         self.assertListRaises(PathError, t.list_dir, 'a')
 
     def test_list_dir_result_is_url_escaped(self):
@@ -1506,7 +1507,10 @@
         transport.put_bytes('foo', 'bar')
         transport3 = self.get_transport()
         self.check_transport_contents('bar', transport3, 'foo')
-        # its base should be usable.
+        # its base should be usable. XXX: This is true only if we don't use
+        # auhentication, otherwise 'base' doesn't mention the password and we
+        # can't access it anymore since the password is lost (it *could* be
+        # mentioned in the url given by the test server) --vila 090226
         transport4 = get_transport(transport.base)
         self.check_transport_contents('bar', transport4, 'foo')
 

=== modified file 'bzrlib/transport/ftp/__init__.py'
--- bzrlib/transport/ftp/__init__.py	2009-01-17 01:30:58 +0000
+++ bzrlib/transport/ftp/__init__.py	2009-02-26 19:30:06 +0000
@@ -63,14 +63,15 @@
 
 
 class FtpStatResult(object):
-    def __init__(self, f, relpath):
+
+    def __init__(self, f, abspath):
         try:
-            self.st_size = f.size(relpath)
+            self.st_size = f.size(abspath)
             self.st_mode = stat.S_IFREG
         except ftplib.error_perm:
             pwd = f.pwd()
             try:
-                f.cwd(relpath)
+                f.cwd(abspath)
                 self.st_mode = stat.S_IFDIR
             finally:
                 f.cwd(pwd)
@@ -145,6 +146,8 @@
                                              port=self._port)
             connection.login(user=user, passwd=password)
             connection.set_pasv(not self.is_active)
+            # binary mode is the default
+            connection.voidcmd('TYPE I')
         except socket.error, e:
             raise errors.SocketConnectionError(self._host, self._port,
                                                msg='Unable to connect to',
@@ -409,7 +412,6 @@
             abspath = self._remote_path(relpath)
             mutter("FTP appe (try %d) to %s", retries, abspath)
             ftp = self._get_FTP()
-            ftp.voidcmd("TYPE I")
             cmd = "APPE %s" % abspath
             conn = ftp.transfercmd(cmd)
             conn.sendall(text)
@@ -530,6 +532,11 @@
                        str(e))
                 return []
             raise
+        finally:
+            # Restore binary mode as nlst switch to ascii mode to retrieve file
+            # list
+            f.voidcmd('TYPE I')
+
         # If FTP.nlst returns paths prefixed by relpath, strip 'em
         if paths and paths[0].startswith(basepath):
             entries = [path[len(basepath)+1:] for path in paths]

